package Net::Braintree::ErrorCodes::MerchantAccount::Business::Address;
use strict;

use constant RegionIsInvalid        => "82684";
use constant StreetAddressIsInvalid => "82685";
use constant PostalCodeIsInvalid    => "82686";

1;
