package Net::Braintree::ErrorCodes::MerchantAccount::AppliantDetails::Address;
use strict;

use constant LocalityIsRequired      => "82618";
use constant PostalCodeIsInvalid     => "82630";
use constant PostalCodeIsRequired    => "82619";
use constant RegionIsRequired        => "82620";
use constant StreetAddressIsInvalid  => "82629";
use constant StreetAddressIsRequired => "82617";
use constant RegionIsInvalid         => "82664";

1;
