package Net::Braintree::ErrorCodes::MerchantAccount::Individual;
use strict;

use constant FirstNameIsRequired    => "82637";
use constant LastNameIsRequired     => "82638";
use constant DateOfBirthIsRequired  => "82639";
use constant SsnIsInvalid           => "82642";
use constant EmailIsInvalid         => "82643";
use constant FirstNameIsInvalid     => "82644";
use constant LastNameIsInvalid      => "82645";
use constant PhoneIsInvalid         => "82656";
use constant DateOfBirthIsInvalid   => "82666";
use constant EmailIsRequired        => "82667";

1;
